/*
 * Copyright 2015 aquenos GmbH.
 * All rights reserved.
 * 
 * Please refer to the file NOTICE.txt in the root of the source distribution or
 * in the META-INF directory of the binary archive for licensing information.
 */

package com.aquenos.epics.jackie.common.value;

/**
 * <p>
 * Channel Access value that represents the
 * {@link ChannelAccessValueType#DBR_TIME_DOUBLE DBR_TIME_DOUBLE} type. This
 * type stores double-precision floating-point numbers and provides information
 * about the alarm status and a time-stamp.
 * </p>
 * 
 * <p>
 * This interface should not be implemented by user-defined classes. Code that
 * wants to create an instance implementing this interface should use the
 * methods provided by {@link ChannelAccessValueFactory} instead.
 * </p>
 * 
 * @author Sebatian Marsching
 */
public interface ChannelAccessTimeDouble extends ChannelAccessAlarmDouble,
        ChannelAccessTimeValue<Double> {

    /**
     * <p>
     * Returns a read-only version of this value. If the value is read-only,
     * this value is returned. Otherwise, a wrapped version of this value, that
     * does not allow modifications is returned.
     * </p>
     * 
     * <p>
     * Please note that the read-only value is not a copy of this value, but
     * just a wrapper around it. This means, that modifications of the wrapped
     * value will result in modifications of the read-only value. Therefore it
     * is bad practice to modify a value that has been used as the base for a
     * read-only value, because code that still has a reference to the read-only
     * value might not expect this change and thus fail unexpectedly.
     * </p>
     * 
     * @return read-only version of this value (never <code>null</code>).
     */
    ChannelAccessTimeDouble asReadOnlyValue();

    /**
     * <p>
     * Creates and returns a copy of this object. The object returned is
     * completely independent from this object. This means that modifications to
     * this object will not affect the returned object and vice-versa.
     * </p>
     * 
     * <p>
     * The object returned is guaranteed to be completely identical to this
     * object, including its type. This means that
     * <code>x.clone().equals(x)</code> and
     * <code>x.clone().getClass() == x.getClass()</code> are always
     * <code>true</code>.
     * </p>
     * 
     * <p>
     * The returned value is guaranteed to allow write access, even if this
     * value is read-only. This means that cloning a read-only value is a good
     * way to get a writable copy.
     * </p>
     * 
     * @return a clone of this instance.
     */
    ChannelAccessTimeDouble clone();

}
